//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "soundserver.h"
#include <qthread.h>
#include <kdebug.h>
#include <errno.h>
#include <unistd.h>
#include <libkbluetooth/scoserversocket.h>
#include <libkbluetooth/scosocket.h>
#include <qfile.h>
#include <qmutex.h>
#include <assert.h>
#include <qevent.h>
#include <qapplication.h>
#include <sys/socket.h>
#include <unistd.h>
#include <fcntl.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/sco.h>
#include <algorithm>
#include "vorbisencoder.h"

using namespace KBluetooth;

class SoundThreadEvent : public QCustomEvent
{
public:
    bool status;

    SoundThreadEvent(bool bStatus) :
        QCustomEvent(QEvent::User+1)
    {
        this->status = bStatus;
    }
};


class SoundThread : public QThread
{
public:
    QMutex vorbisEncoderMutex;

    SoundThread(SoundServer *owner, int socket)
        //localSoundFile("/tmp/local_sound"),
        //remoteSoundFile("/tmp/remote_sound")
    {
        this->owner = owner;
        //localSoundFile.open(IO_WriteOnly);
        //remoteSoundFile.open(IO_WriteOnly);
        this->scoSocket = socket;
        bCont = true;
    }

    ~SoundThread()
    {
        bCont = false;
        wait();
        localSoundFile.close();
        remoteSoundFile.close();
    }

protected:
    virtual void run()
    {
        if (!scoSocket) return;
        int socket = scoSocket;
        // Assume 16 Bit sample Size (e.g. voice setting 0x0060)
        typedef Q_INT16 SampleType;
        const int sampleBytes = sizeof(SampleType);

        QApplication::postEvent(owner, new SoundThreadEvent(true));
        arts_init();
        recordStream = arts_record_stream(8000, sampleBytes*8, 1, "kbthandsfree/record");
        playStream = arts_play_stream(8000, sampleBytes*8, 1, "kbthandsfree/play");

        const int data_size = 672;
        const int data_count = data_size/sampleBytes;
        int len;
        int write_len;
        static SampleType buf1[data_count];
        static SampleType buf2[data_count];

        arts_stream_set(playStream, ARTS_P_BUFFER_SIZE, data_size);
        arts_stream_set(recordStream, ARTS_P_BUFFER_SIZE, data_size);

        arts_stream_set(playStream, ARTS_P_BLOCKING, 0);
        arts_stream_set(recordStream, ARTS_P_BLOCKING, 1);

        double origSampleAccu = 0.0;
        double swappedSampleAccu = 0.0;
        kdDebug() << "Started sound thread" << endl;
        kdDebug() << "::read" << endl;
        while (bCont && ((len = ::read(socket, buf1, data_size)) > 0 || (errno==EAGAIN))) {
            if (len<=0) {
                kdDebug() << "SCO: read()==EAGAIN" << endl;
                msleep(5);
                continue;
            }
            assert(len%2==0);
            //remoteSoundFile.writeBlock((char*)buf1, len);

            // Sometimes the byte order of the incoming sco data seems to
            // be wrong, or maybe there is an offset in the audio data.
            // This happens with CSR USB dongles and the Nokia 3650 at least..
            //
            // This is a dirty workaround for that problem. We try to guess
            // the byte order and swap the byte order if this seems neccessary.
            for (int n=0; n<data_count; ++n) {
                Q_UINT16 usample = Q_UINT16(buf1[n]);
                double orig = abs(Q_INT16(usample));
                //orig *= orig;
                double swapped = abs(Q_INT16(((usample&0xFF00)>>8) + ((usample&0xFF)<<8)));
                //swapped *= swapped;
                origSampleAccu += orig;
                swappedSampleAccu += swapped;
            }
            if (swappedSampleAccu < origSampleAccu) {
                kdWarning() << "offset wrong!" << endl;
                for (int n=0; n<data_count-1; ++n) {
                    Q_UINT16 usample1 = Q_UINT16(buf1[n]);
                    Q_UINT16 usample2 = Q_UINT16(buf1[n+1]);
                    buf1[n] = Q_INT16(((usample2&0xFF00)>>8) + ((usample1&0xFF)<<8));
                }
                buf1[data_count-1] = buf1[data_count-2];
            }

            int artsreadlen = arts_read(recordStream, buf2, len);
            // TODO: Mix recorded stream into played stream if requested
            //memcpy(buf1, buf2, data_count);
            arts_write(playStream, buf1, len);

            //kdDebug() << QString("arts_read len=%1)").arg(artsreadlen) << endl;
            if (len>0) {
                assert(artsreadlen%2==0);
                //kdDebug() << "::write" << endl;
                write_len = ::write(socket, (char*)buf2, artsreadlen);
                //localSoundFile.writeBlock((char*)buf2, artsreadlen);
            }
            else {
                kdWarning() << QString("arts_read failed (err=%1)").arg(artsreadlen) << endl;
                break;
            }

            // Now write out the data as a vorbis file
            {
                QMutexLocker locker(&vorbisEncoderMutex);
                if (owner->vorbisencoder) {
                    owner->vorbisencoder->writeBlock(buf1, buf2, std::min(len,artsreadlen)/2);
                }
            }

            //"Recevied and sent %d bytes (write: %d)\n", len, write_len);
            //kdDebug() << "::read" << endl;
        }
        kdDebug() << QString("Sound thread terminated (errno=%1)").arg(errno) << endl;
        ::close(socket);

        arts_close_stream(playStream);
        arts_close_stream(recordStream);
        arts_free();
        QApplication::postEvent(owner, new SoundThreadEvent(false));
    }

private:
    QFile localSoundFile;
    QFile remoteSoundFile;
    arts_stream_t playStream;
    arts_stream_t recordStream;
    SoundServer *owner;
    int scoSocket;
    bool bCont;
};


// --- class SoundServer ---

SoundServer::SoundServer(QObject *owner) :
    QObject(owner)
{
    bConnected = false;
    soundThread = NULL;
    vorbisencoder = NULL;
    serverSocket = new ScoServerSocket(this);
    kdDebug() << "Init SCO" << endl;
    connect(serverSocket, SIGNAL(onNewConnection(int, KBluetooth::DeviceAddress)),
        this, SLOT(slotNewScoConnection(int, KBluetooth::DeviceAddress)));
}

SoundServer::~SoundServer()
{
    if (soundThread) {
        delete soundThread;
    }
}

void SoundServer::slotNewScoConnection(int socket, KBluetooth::DeviceAddress /*addr*/)
{
    if (soundThread) {
        delete soundThread;
    }
    soundThread = new SoundThread(this, socket);
    soundThread->start();
}


bool SoundServer::event(QEvent *e) {
    if (e->type() == QEvent::User + 1) {
        // Sound Thread event
        SoundThreadEvent *se = (SoundThreadEvent*)e;
        bConnected = se->status;
        kdDebug() << "SoundServer status update: " << bConnected << endl;
        emit speechConnectionStatus(se->status);
        return true;
    }
    return false;
}

void SoundServer::openSpeechConnection(KBluetooth::DeviceAddress addr)
{
    if (connected()) return;

    if (soundThread) {
        delete soundThread;
    }
    soundThread = NULL;

    int scoSocket;

    sockaddr_sco rem_addr, loc_addr;
    if( (scoSocket = ::socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_SCO)) < 0 ) {
        kdDebug() << QString("Can't create socket. %1(%2)")
            .arg(strerror(errno)).arg(errno) << endl;
        return;
    }

    memset(&loc_addr, 0, sizeof(loc_addr));
    loc_addr.sco_family = AF_BLUETOOTH;
    loc_addr.sco_bdaddr = DeviceAddress::any.getBdaddr();
    if (bind(scoSocket, (struct sockaddr *) &loc_addr, sizeof(loc_addr)) < 0 ) {
        kdDebug() << QString("Can't bind socket. %1(%2)")
            .arg(strerror(errno)).arg(errno) << endl;
        return;
    }

    memset(&rem_addr, 0, sizeof(rem_addr));
    rem_addr.sco_family = AF_BLUETOOTH;
    rem_addr.sco_bdaddr = addr.getBdaddr();
    if(::connect(scoSocket, (struct sockaddr *)&rem_addr, sizeof(rem_addr)) < 0 ) {
        kdDebug() << QString("Can't connect. %1(%2)")
            .arg(strerror(errno)).arg(errno) << endl;
        ::close(scoSocket);
        return;
    }
    kdDebug() << "Connected." << endl;

    soundThread = new SoundThread(this, scoSocket);
    soundThread->start();
}

void SoundServer::closeSpeechConnection()
{
    if (soundThread) {
        delete soundThread;
    }
    soundThread = NULL;
}

bool SoundServer::connected()
{
    return bConnected;
}

void SoundServer::setVorbisEncoder(VorbisEncoder* encoder)
{
    if (soundThread) {
        QMutexLocker locker(&soundThread->vorbisEncoderMutex);
        this->vorbisencoder = encoder;
    }
    else {
        this->vorbisencoder = encoder;
    }
}

#include "soundserver.moc"
